--------------------------------------------------------------------------------
-- Copyright  2021								|
-- All Rights Reserved.								|
-- 										|
-- Permission to use, copy, modify, and distribute this software in any way	|
-- is not granted without expressed written agreement. This copyright applies	|
-- to the all code, data and artwork related to this project and all associated	|
-- projects published by the author.						|
--										|
-- More information on copyright is available from sources on-line.		|
-- Use an actual factual one!							|
--------------------------------------------------------------------------------

	------------------------------------------------------------------------------------------------------------
	--				  Init. Game Assets					  	  --
	------------------------------------------------------------------------------------------------------------

local AddonName, NS = ...
local L = NS.TEXTS

local upper = string.upper
local AddOnInfo = (C_AddOns and C_AddOns.GetAddOnInfo) and C_AddOns.GetAddOnInfo or GetAddOnInfo

function NS.CopyTable(source, target)
	for key, value in pairs(source) do
		if type(value) == "table" then
			NS.CopyTable(value, target[key])
		else
			target[key] = value
		end
	end
end

local AltNames = {
	AutoBar = { "AutoBarClassic", "AutoBarBCC", "AutoBarWrath" },
}

local Exclusions = {
	METERS = true,
	RAIDFRAMES1 = true,
	COMBATLOG = true,
}

local ValidReason = {
	DEMAND_LOADED = true,
	DISABLED = true,
}

local function GetAddonExists(addon)
	local name, title, notes, loadable, reason = AddOnInfo(addon)
--	if not reason or strtrim(reason) == "" or ValidReason[reason] then
	if not reason or strtrim(reason) == "" or ValidReason[reason] then
		return addon
	end
	if not AltNames[addon] then
		return
	end
	for i=1, #AltNames[addon] do
		name, title, notes, loadable, reason = AddOnInfo(AltNames[addon][i])
		if not reason or strtrim(reason) == "" or ValidReason[reason] then
			return addon, AltNames[addon][i]
		end
	end
end

local function InitialiseAddons(self, addon)
	local Settings = NS:GetSettings("Addons")
	local Defaults = NS:GetSettings().Defaults.Addons
	for k, settings in pairs(Settings) do
		local AddName, AltName, Ignore
		if Exclusions[upper(k)] then
			AddName = k
			Ignore = true
		else
			AddName, AltName = GetAddonExists(k)
			if not AddName then
				AddName = k
			end
		end
--		if AddName and (not addon and not Exclusions[upper(k)] or addon == k) and not settings.inactive then
		if AddName and not settings.inactive then
			if not Ignore then
				if not IsAddOnLoaded(AltName or AddName) then 
					settings.inactive = true
				end
			end
			-- Should get rid of the RunScript and replace with straight function one day
			-- See: Dominos and DUF \Addon (not \Addon\Config) .lua code in FauxMazzleConfig.
			if Defaults[k] and Defaults[k].configfunc then
				RunScript("function FauxMazzleRun(CopyTable, settings)\n"..Defaults[k].configfunc.."\nend")
				if FauxMazzleRun then
					FauxMazzleRun(NS.CopyTable, settings)
				end
				-- or replace above with
				-- Defaults[k].configfunc(CopyTable, settings)
			end
		else
			if not AddName then
				settings.inactive = true
			end
		end
	end
	if FauxMazzleRun then 
		FauxMazzleRun = nil 
	end
end

NS:RegisterForInit("PLAYER_LOGIN", "InitialiseAddons", 3, InitialiseAddons)
NS:AddOption("InitialiseAddons", InitialiseAddons)
